; GRABTIM6.ASM  --  ATTINY2313 micro-controller project

; Copy of GrabTim5 with exception that relay drive is direct
;    insteady of through 100 uf capacitors.  Lowered LED brightness
;    is the result to assure voltage does not sink below 5.0 volts


; ***** / Grabs time, stores it and executes commands repeatedly

; Fuse set for Xtal 3.579545, 65 ms
;   3-8 Mz, 14CK, 65 ms, CKSEL=1101, SUT 11
;   (slowly rising power)

; March 5, 2007  - put in Belkin computer power box.

.include "tn2313def.inc"

; R16 general purpose temporary use register
; R18 is flags register
; R20 contains downcnt.
; R21 contains TPCounter count
; R22 is ten counter (Interrupt Routine)


;*****Initialization


rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt

	
START1:			;set up stack pointer and initialize cold start

ldi r16,low(RAMEND)	; Load low byte address of end of RAM into register R16
out	SPL,R16			; Initialize stack pointer to end of internal RAM

ser	r16			;Set to all ones for out port direction as output
out	DDRB,r16	;Set PORTB to output

clr	r16
out	PortB,r16	;LEDs are off when port pins are low

ldi	R16,251		;Set bit 2(4) as input ... remaining bits as output
out	DDRD,R16	;execute data direction with bit 0 as input
out	PORTD,R16	;execute to set out ports high, and in port with
				; no input resistor

in  r16,TIMSK   ;enable 16-bit match to A register
ori r16,64
out TIMSK,r16

in  r16,TCCR1B
ori r16,11          ; divide by 64 on prescaler
out TCCR1B,r16      ;enable reset of counter to zero (bit3) after match detection.
                    ;and bits 0,1,2 select prescaling source as 64 of clock

                    ;3,579,545 Xtl with 8 division = 447,443
                    ;  divided by 64 = 6991.3
ldi  r16,27          ;comparator high byte for this crystal
out  OCR1AH,r16
ldi  r16,79          ;comparator low byte for this crystal 3/27/2006
out  OCR1AL,r16      ;for one second intervals

			
rcall	ZeroCommandMemory		;zero all Command Memory locations


clr	r18		;zero all flags in register r18
sts	$68,r18	;zero delay on time minute store area


InitializeClkTime:		;starts clock at 8:00 a.m.

ldi	r16,1
sts	$63,r16			;put Sunday(1) in as start of day-of-week

ldi	R16,224			;8:00 A.M. low byte
sts	$61,r16		

ldi	r16,1			;8:00 A.M. high byte
sts	$62,R16

mov	r15,r16			;set seconds counter to one


ldi	r16,25			;add 23 seconds for clock correction
sts	$64,r16			;default load of clock correction in seconds

					;old figures = 23 secs in 10 days
					;Feb 16, 2007 = 25 secs in 12 days - (grabtime5b)

ldi	r16,12			;make this correction every 10th day
sts	$65,r16			;store in SRAM - permanent store, every other day, etc.
sts	$66,r16			;store in SRAM - temporary store

	
ldi	r26,$006A		;initialize at start of memory locations
				
ldi	r16,$6C			;set up start of recording area for data
sts	$67,r16			;bookmark start of command memory


bset	7			;enable global interrupts

;------------------------------------


MainLoop:

sbic	PIND,2		;skip if switch down, PIND,2(4)
rjmp	ML1a

rcall	DetectCntTwo


ML1a:

sbrc	r18,3		;skip if bit 3 (8), grabtime is disabled
rjmp	ML1d

mov		r16,r15			;check for 00 of seconds counter
cpi		r16,00
breq	ML1

cpi		r16,30			;check for 30 seconds into count
breq	ML3
rjmp	MainLoop


ML3:

ldi		r16,2		;number of wag flashes
rjmp	ML1b


ML1:

ldi		r16,8


ML1b:

rcall	VarWagRGinterruptible
rcall	IDelayHalf
rcall	IDelayHalf	;delay before flashes start
rcall	DisplayClock

rjmp	MainLoop	;circle back to repeat loop


ML1d:			;GrabTime is enabled -- flash red LED every second

sbrs	r18,2		;skip if seconds flag is set, r18,2(4)
rjmp	MainLoop

cbr		r18,4		;clear seconds flag, r18,2(4)
sbi		PortB,0		;on red LED
rcall	Delay100th
cbi		PortB,0		;off red LED

rjmp	MainLoop

;---------------------------


Button3:		;Replaces old DetectCnt  2/15/2006

sbrc	r18,3			;skip if bit 3 (8), grab time clear
rjmp	B1continue		;grab time is in record mode

rcall	ToggleRelayRecord
clr		r20
clr		r21				;zero out any commands

ret


B1continue:

push	r23

clr		r20		;HoldDown count
clr		r21		;short touch count 1-9
clr		r23		;clear countdown register


B1:

inc		r23
rcall	Delay40th
cpi		r23,60		;delay before first holddown count - 40=one second
brsh	B3			;branch if - it is a holddown count

sbis	PIND,2		;skip if switch is NOT down
rjmp	B1


B2b:				;Count is detected

inc		r21			;add 1 minute to r21

rcall	Delay40th
clr		r23			;clear r23


B2d:

sbis	PIND,2		;skip if switch is up
rjmp	B2e			;jump back - there is a new entry coming in

inc		r23
rcall	Delay40th
cpi		r23,60		;has switch been up 60 counts? 40=one second 
brsh	B5

rjmp	B2d


B2e:

clr		r23
rjmp	B1


B3:					;Holddown count has been detected

inc		r20			;increment holddown count
sbi     PortB,0
rcall   Delay40th
cbi     PortB,0         ;flash PROGRAM LED


clr		r23			;reset r23 to zero count


B4:

inc		r23
rcall	Delay40th
sbic	PIND,2		;skip if switch is down
rjmp	B4a

cpi		r23,30		;hold down count length between flashes
brlo	B4

rjmp	B3


B4a:

rcall	Delay40th


B5:

pop		r23
ret


;------------------------

Button3Delay:		;waits until switch is pressed

sbic	PIND,2		;skip if switch is down
rjmp	Button3Delay
rjmp	Button3


;------------------------


DetectCntTwo:		;includes string of functions requiring attention

push	r16

rcall	Button3	
	
			;---- now comes the decoding

cpi		r20,1		;checks to see if LONG Count has been recorded
brsh	LongCount	;branch if same or higher than 1

; The following are short counts


ShortCount:


SC2:			;two short depresses store every day time command

cpi		r21,2		;#1 short = Togle AC on/off
breq	SC2a
rjmp	SC3


SC2a:

clr		r16
sts		$6B,r16		;zero is flag for every day command
rcall	GrabTime
rcall	ToggleAC	;toggles relay

rjmp	LCCEnd

;------------------------------------

SC3:			;three touches grab day-of-week in addition

cpi		r21,3
breq	SC3a
rjmp	LCCEnd


SC3a:

ldi		r16,1
sts		$6B,r16		;one equals add day of week information
rcall	GrabTime
rcall	ToggleAC

rjmp	LCCEnd

;-----------------------------


LongCount:		;series for long counts


;---------------------------

LCC3:

cpi		r20,3		;Puts 17 commands into command memory, erasing old
				; commands.  Both summer and winter series can be
				; programmed.
				
breq	LCC3a
rjmp	LCC4


LCC3a:

ldi		r16,1		;set up for ONE green flash
rcall	FlashGLED	;indicate that one entry is needed

rcall	Button3Delay
cpi		r20,8		;if HC8 then proceed for summer times
breq	LCC3b

cpi		r20,9		;if HC9 then proceed for winter times
breq	LCC3c	
rjmp	LCCEnd


LCC3b:

rcall	OnTimes		;These are summer hours.  Starts at 8:00 p.m.
			; running until about 11:00 p.m.  On off cycle
			; is ten minutes for 17 cycles.
rjmp	LCCEnd


LCC3c:

rcall	OnTimes2	;These are winter hours.  Starts at 6:00 p.m.
			; running until about 11:00 p.m.  On off cycle
			; is seventeen minutes for 17 cycles.
rjmp	LCCEnd


;-----------------------------

LCC4:

cpi		r20,4		;Turns off GRAB TIME program mode
breq	LCC4a
rjmp	LCC5


LCC4a:

cbr		r18,8		;clear bit 3(8) - off record mode flag
rjmp	LCCEnd


;------------------------------


LCC5:			;Clear command memory (EEPROM not affected)

cpi		r20,5	
breq	LCC5a
rjmp	LCC6


LCC5a:			;Clear command memory

rcall	ZeroCommandMemory
rjmp	LCCEnd

;-----------------------------


LCC6:

cpi		r20,6		;Manually enters time into command memory
breq	LCC6a
rjmp	LCC7


LCC6a:

rcall	ManualProgramEntry

rjmp	LCCEnd


;------------------------

LCC7:				;Set clock

cpi		r20,7
breq	LCC7a
rjmp	LCC8


LCC7a:

rcall	SetClock
rjmp	LCCEnd


;------------------------------

LCC8:			;Writes commands to EEPROM
			
cpi		r20,8	
breq	LCC8a
rjmp	LCC9			


LCC8a:

rcall	writeEEPROM
rjmp	LCCEnd

;-------------------------------


LCC9:			;Reads commands from EEPROM memory
			
cpi		r20,9
breq	LCC9a
rjmp	LCC10


LCC9a:

rcall	ReadEEPROM
rjmp	LCCEnd

;---------------------------------


LCC10:			;Set LSB for compare register 3.58 countdown

cpi		r20,10
breq	LCC10a
rjmp	LCC11


LCC10a:

rcall	SetCompareRegister	;set 3.58 clock compare register during interrupts

rjmp	LCCEnd


;--------------------------------

LCC11:

cpi		r20,11		;Set number of seconds correction and how often
breq	LCC11a
rjmp	LCCEnd


LCC11a:

rcall	SecondsCorrection	;set correction
rjmp	LCCEnd

;------------------------------


LCCEnd:

rcall	TwoWags
rcall	Delay20th

pop		r16

ret

;---------------------------------------


ClockCntDwnInt: 		;interrupt to update clock counters

push	r16

in		r16,SREG	;save flags
sts		$60,r16

sbr		r18,4		;set bits 2 (4) -- second flag 
					;(set each second)

inc		r15			;step one count of SECONDS counter
mov		r16,r15

cpi		r16,3		;if 3 then do minutes on routine for light
brne	CCDITimeBypass

push	r16
lds		r16,$68		;get time on value in $68
or		r16,r16
breq	CCDITimeBypass1

inc		r22			;increment r22, ten counter
cpi		r22,10
brne	CCDITimeBypass1

clr		r22			;zero ten counter

dec		r16
sts		$68,r16

cpi		r16,0
brne	CCDITimeBypass1

rcall	ToggleAC


CCDITimeBypass1:

pop		r16


CCDITimeBypass:

cpi		r16,60
breq	CCDI1

rjmp	CCDIExit		;return since no action required


CCDI1:

clr		r15			;zero r15 -- SECONDS counter

sbr		r18,16		;set bit 4 (16) in R18 -- the NEW MINUTE flag


CCDI1Cntr:

lds		r16,$61		;load low minutes byte into R16

inc		r16
sts		$61,r16

brne	CCDI1a		;jump out if not zero (no overflow)

lds		r16,$62		;high byte minutes
inc		r16			;increment high byte minutes
sts		$62,r16


CCDI1a:

lds		r16,$62		;check high byte if it is 5
cpi		r16,5

breq	CCDI1b

rjmp	CCDIalarmExit


CCDI1b:			

lds		r16,$61
cpi		r16,160		;1440 minutes reached -- it's now midnight

breq	CCDI1c

rjmp	CCDIalarmExit


CCDI1c:			;zero high and low bytes - it is midnight

clr		r16
sts		$61,r16
sts		$62,r16

lds		r16,$63
inc		r16
sts		$63,r16

cpi		r16,8

breq	CCDI1d


CCDI1e:

rcall	ClockCorrection	;call clock correction (midnight corr)

rjmp	CCDIalarmExit


CCDI1d:

ldi		r16,1		;load with 1 (Sunday)
sts		$63,r16
rjmp	CCDI1e		;to continue


CCDIalarmExit:

rcall	CheckAllAlarms	



CCDIExit:		;Exit this interrupt routine

lds		r16,$60		;SREG temporary store
out		SREG,r16	;restore SREG data

pop		r16

reti


;-----------------------------


CheckAllAlarms:		;check entire list of alarm commands for match

push	r14		;temporary x register store
push	r25
push	r26
push	r28
push	r29
push	r30
push	r31


ldi		r26,$6C		;low byte -- start of alarm memory - x register
mov		r14,r26


CAAc:				;check for end of list

mov		r26,r14		;restore x register value to r26
lds		r16,$67		;get present bookmark for command memory pointer
cp		r26,r16
brlo	CAAa
rjmp	CAAEnd		;jump out of loop if equal or higher (end of list)


CAAa:			;Is there a time match?

ld		r30,x+		;load low command byte into r30 and post decrement
ld		r31,x+		;load high command byte into r31 and post decrement
mov		r14,r26		;save x register location in r14
mov		r25,r31
andi	r25,248		;save high bytes for later processing in r25

lds		r28,$61		;get minutes low and put in r28
lds		r29,$62		;get minutes high and put in r29

andi	r31,7		;retain only low bits 0,1,2 of high byte in r31

cp		r28,r30		;low byte compare
breq	CAAa1		;do further comparing

rjmp	CAAc		;go for another round of search


CAAa1:

cp		r29,r31		;high byte compare
breq	CAAb

rjmp	CAAc		;go for another round of search


CAAb:			;do further checks on high bytes of MSD

tst		r25		;check for command bit in r25 for all being zero
				; (every day command)
breq	CAAf		;toggle relay


CAAk:			;it's a day-of-week command

lsr		r25
lsr		r25
lsr		r25		;shift day of week into position

lds		r16,$63
cp		r25,r16
breq	CAAf		;toggle relay
rjmp	CAAc		;go for another round


CAAf:			;toggle relay and return to next check

rcall	ToggleAC	;toggle relay
rjmp	CAAc


CAAEnd:

pop		r31
pop		r30
pop		r29
pop		r28
pop		r26
pop		r25
pop		r14

ret

;-------------------------------------------------------


ClockCorrection:	;subtracts seconds in $66 to time at minight 
			
lds		r16,$66
tst		r16		;if zero, no correction is needed
			;1= every day, 2= every other day, etc.

breq	CCExit		;exit routine with no action

dec		R16
sts		$66,r16		;store new value in variable

breq	CC3		;it is time for update of second counter

ret


CC3:

lds		r16,$64		;seconds correction from permanent store
mov		r15,r16		;put in seconds store area to correct time

lds		r16,$65		;get permanently stored value for interval correction
sts		$66,r16		;store new value in variable


CCExit:

ret

;----------------------------------------------


VarClockFlash:		;flashes red or green LED (G=AM R=PM)
					;R16 contains number of flashes
push	r17

rcall	IDelayHalf	;Delay before display of time

lds		r17,$62		;check if AM or PM time (less than 720 = AM)
cpi		r17,2		;go to AM if high byte is <2 
brlo	VCF1		;jump to AM routine
cpi		r17,2
breq	VCF3
rjmp	VCF2		;jump to PM routine


VCF3:

lds		r17,$61		;compare low byte for less than 208
cpi		r17,208		
brlo	VCF1		;jump to AM routine
rjmp	VCF2		;jump to PM routine


VCF1:				; AM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

sbi		PortB,1		;turn on GREEN LED (AM)
rcall	IDelay40th	;on time
cbi		PortB,1		;off Green LED

sbrc	r18,1		;skip if bit 1 is clear - no double flash
rjmp	VCF1ab		;go for double flash


VCF1ac:

rcall	IDelayQuarter		;off time
dec		r16
rjmp	VCF1


VCF1ab:				;gives double flash

rcall	IDelay20th	;small pause
sbi		PortB,1		;on green LED
rcall	IDelay40th	;on time
cbi		PortB,1		;off Green LED
rjmp	VCF1ac


VCF2:				; PM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

sbi		PortB,0		;turn on RED LED
rcall	IDelay40th	;on time delay
cbi		PortB,0		;turn off RED LED
sbrc	r18,1		;skip if bit 1 is clear - no double flash
rjmp	VCF2ab


VCF2ac:

rcall	IDelayQuarter		;off time delay
dec		r16
rjmp	VCF2


VCF2ab:				;give double red flash

rcall	IDelay20th	;short off delay
sbi		PortB,0		;on Red LED
rcall	IDelay40th	;on time delay for second flash
cbi		PortB,0		;off Red LED
rjmp	VCF2ac


VCF2a:

pop		r17
cbr		r18,2		;clear bit 1 (2) - double flash flag
ret

;-------------------------------------------


DisplayClock:	;displays time

clr		r6		;zero hours count
clr		r7		;zero MSD of minutes count

bclr	7		;turn off interrupts

lds		r28,$61		;load low byte into r28
lds		r29,$62		;load high byte into r29

bset	7		;turn interrupts back on

tst		r29		;test high byte for zero
brne	DC3

cpi		r28,60		;check to see if less than 60 (12:00 am to 12:59 am)
brlo	DC2
rjmp	DC3


DC2:

ldi		r16,12		;flash out 12 a.m.

rcall	VarClockFlash	;flash out time of 12 a.m.

rjmp	DC7


DC3:				;flash out other times

mov		r4,r28		;save low byte in r4
mov		r5,r29		;save high byte in r5

inc		r6			;increment hours count
sbiw	r28,60		;subtract 60 from the r28,29 pair

brcs	DC3a		;hours now almost decoded

rjmp	DC3			;repeat loop again


DC3a:

dec		r6		;restore old value of hours
mov		r28,r4		;restore low byte in r28
				;at this point, minutes remainder is in low byte
				; (r29 will be zero)
			

DC10:

mov		r16,r6		;put hours into r16 for check
cpi		r16,13
brsh	DC9
rjmp	DC8


DC9:

subi	r16,12


DC8:			;flash out time

rcall	VarClockFlash


DC7:			;decode minutes with MSD double flashed

mov		r16,r28
inc		r7		;use r7 for MSD count
subi	r28,10	;get remaining minutes
brcs	DC7a

rjmp	DC7		;loop back for another subtraction


DC7a:

dec		r7			;restore correct value for MSD
mov		r28,r16		;restore correct remainder (LSDs)

mov		r16,r7		;put in MSD for flashout
sbr		r18,2		;set bit 1 (2) for double flash
rcall	VarClockFlash	;flash out MSD
cbr		r18,2		;clear bit 1 (2) of double flash

mov		r16,r28
rcall	VarClockFlash	;flash out LSDs

ret


;---------------------------


ZeroCommandMemory:	;zero all command storage locations

push 	r1
		
clr		r1
ldi		r26,$6C	;low start byte for start of storage locations
sts		$67,r26	;store new bookmark for start of entries

ldi		r16,68	;for zeroing 64 locations

ZCM:

st		x+,r1	;zero location

dec		r16	
brne	ZCM

pop		r1
ret

;----------------------------------


ToggleAC:			;toggles AC power on/off

sbrs	r18,6		;skip if r18,6(64) is set -- relay state is on
rjmp	OnRelay


OffRelay:

sbi		PortB,7		;on bit 7 for off pulse
rcall	Delay80th
cbi		PortB,7		;off bit 7 to complete off pulse
cbr		r18,64		;relay state off, r18,6(64)
ret


OnRelay:

sbi		PortB,6		;set bit 6 for on pulse
rcall	Delay80th
cbi		PortB,6		;off bit 6 to complete on pulse
sbr		r18,64		;relay state to on, r18,6(64)

ret


;--------------------------


SetCompareRegister:

		;puts new value into compare register A.  
		;Default = 218 for high byte, 140 for low byte.
		
push	r1

clr		r1
rcall	TwoGreenFlash
rcall	OneGreenFlash	;total of three flashes indicating multiple inputs

rcall	Button3Delay
rcall	Mult100		;multiply r20 value by 100
mov		r1,r20		;set 100's column

rcall	OneGreenFlash	;indicate an entry is needed
rcall	Button3Delay
rcall	Mult10		;multiply r20 value by 10
add		r1,r20		;add in tens units

rcall	OneGreenFlash	;indicate an entry is needed
rcall	Button3Delay
add		r1,r20		;add in LSD of count -- data is now complete

out		OCR1AL,r1	;put new compare register A data in place

pop		r1
ret

;-------------------------------------


SecondsCorrection:	;Set seconds of correction and how often (2 entries)

rcall	TwoGreenFlash
rcall	Button3Delay
sts		$64,r20		;seconds of correction (zero to 255)

rcall	OneGreenFlash
rcall	Button3Delay	;enter interval from 0 to 255 - zero = no correction
sts		$65,r20
sts		$66,r20			;put value both in perm and temp store areas

ret

;---------------------


SetClock:		;sets clock for AM/PM, hour, minutes, and day-of-week
			;  24 hour time entry required

push	r1		;minutes accumulator
push	r2		;hours accumulator
push	r3		;day of week accumulator
push	r26
push	r27

clr		r26		;low byte accumulator
clr		r27		;high byte accumulator

ldi		r16,5		;set up for FIVE green flashes
rcall	FlashGLED	;indicate that five entries are needed

rcall	Button3Delay
mov		r3,r20		;day of week (1=Sunday, 7=Saturday)

rcall	TwoGreenFlash 	;Two green flashes indicate two entires MSD,LSD
rcall	Button3Delay	;enter MSD of hours 

rcall	Mult10		;multiply value by 10 and store in r2
mov		r2,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	Button3Delay	;enter LSD of hours
add		r2,r20		;value in hours now complete in r2
	
rcall	TwoGreenFlash	;signal another two entries needed
rcall	Button3Delay	;enter msd of minutes
rcall	Mult10		;multiply by 10 and store in r1
mov		r1,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	Button3Delay
add		r1,r20		;value in minutes now complete


SetC1:

tst		r2		;total hours count
breq	SetC2		;done
adiw	r26,60
dec		r2
rjmp	SetC1


SetC2:

add		r26,r1		;total minutes count
brcs	SetC3
rjmp	SetEnd


SetC3:

inc		r27		;carry flag was set
rjmp	SetEnd


SetEnd:

bclr	7		;turn off global interrupts

sts		$63,r3		;store day-of-week
sts		$61,r26		;store low byte in $61
sts		$62,r27		;store high byte in $62
clr		r15		;zero seconds counter

bset	7		;turn on global interrupts

pop		r27
pop		r26
pop		r3
pop		r2
pop		r1

ret


;--------------------------------------


;FLASHRLED - flashes Red LED No. of Times in r16 (non-interruptible)

FlashRLED:

push	r0
mov		r0,r16

	
FlashRLEDa:

tst		r16
breq	FlashRLEDend
						;put value of R16 in R0
sbi		PORTB,0			;Turn on RED LED
rcall	Delay40th
cbi		PORTB,0			;Off RED LED
rcall	DelayHalf	
dec		r0
brne	FlashRLEDa


FlashRLEDend:

pop		r0

ret

;-----------------------

FlashGLED:		;Flash Green LED no. of time in R16 (non-interruptible)

push	r0
mov		r0,r16

tst		r16
breq	FlashGLEDend


FlashGLEDa:

rcall	DelayHalf
sbi		PORTB,1			;Turn on GREEN LED
rcall	Delay40th
cbi		PORTB,1			;Off GREEN LED
dec		r0
brne	FlashGLEDa


FlashGLEDend:

pop		r0

ret

;------------------------------------


OneGreenFlash:

ldi		r16,1
rcall	FlashGLED
ret

;------------------------------------


TwoGreenFlash:

ldi		R16,2		;prepare for two flashes
rcall	FlashGLED
ret

;-------------------------------------


TwoRedFlash:

ldi		R16,2		;prepare for two flashes
rcall	FlashRLED	
ret

;------------------------------------


RapidRedFlashInterruptible:	;r16 contains number of rapid red flashes

lsl		r16			;double count in r16 for this flashout

RRF1:

sbi		PortB,0		;on red LED
rcall	IDelay40th
cbi		PortB,0		;off red led
rcall	IDelay10th

dec		r16
brne	RRF1

ret	


;-------------------------------------


IGFlash:			;Interruptible green flash - R16  contains 
				; number of flashes

tst		r16
breq	IGF1
sbi		PortB,1		;on Green LED
rcall	IDelay40th
cbi		PortB,1		;off Green LED
rcall	IDelayHalf
dec		r16
rjmp	IGFlash


IGF1:
rcall	IDelayHalf
ret				

;----------------------------


IRFlash:			;Interruptible red flash - R16  contains 
				; number of flashes

tst		r16
breq	IRF1
sbi		PortB,0		;on Green LED
rcall	IDelay40th
cbi		PortB,0		;off Green LED
rcall	IDelayHalf
dec		r16
rjmp	IRFlash


IRF1:
rcall	IDelayHalf
ret				


;---------------------------


VarWagRGInterruptible:		;R16 contains number of wags

sbi		PortB,0
rcall	IDelay40th
cbi		PortB,0
rcall	IDelay20th
sbi		PortB,1
rcall	IDelay40th
cbi		PortB,1
rcall	IDelay20th
dec		r16
brne	VarWagRGInterruptible

ret

;-------------------------------


VarWagRG:		;R16 contains number of wags

sbi		PortB,0
rcall	Delay40th
cbi		PortB,0
rcall	Delay20th
sbi		PortB,1
rcall	Delay40th
cbi		PortB,1
rcall	Delay20th
dec		r16
brne	VarWagRG

ret

;-----------------------


TwoWags:		;do two wags of the VarWagRG

ldi		r16,2

rcall	VarWagRG
ret

;-----------------------

;Delay1 subroutines

;-------------------------


Delay100th:				; 1/100th second delay

push	r0
push	r31

clr		r0
ldi		r31,5

rjmp	D40a

;-----------------------------


Delay80th:

push 	r0
push	r31

clr		r0
ldi		r31,7

rjmp	D40a


Delay40th:              ; 1/40th second delay

push    r0
push    r31

clr     r0
ldi     r31,14          ;put 16 into r31 (high byte) for countdown


D40a:

dec     r0
brne    D40a

dec     r31              ;freq countdown = 1/40th second
brne    D40a

pop     r31
pop     r0

ret

;--------------------------


Delay20th:

push	r16
ldi		r16,2


Delay20th1:

rcall	Delay40th
dec		r16
brne	Delay20th1

pop		r16
ret

;-------------------------------


Delay10th:

push	r16
ldi		r16,4


Delay10th1:

rcall	Delay40th
dec		r16
brne	Delay10th1

pop		r16
ret

;---------------------------------


DelayHalf:		;1/2 second delay

push	r16
ldi		r16,20


DelayHalf1:

rcall	Delay40th
dec		r16
brne	DelayHalf1

pop		r16
ret

;------------------------


IDelay40th:              ; 1/40th second delay

push    r0
push    r31

clr     r0
ldi     r31,14          ;put 16 into r31 (high byte) for countdown


ID40a:

sbis	PIND,2			;skip if switch is up, PIND2(4)
rjmp	IDelay40b

dec     r0
brne    ID40a

dec     r31              ;freq countdown = 1/40th second
brne    ID40a

rjmp	IDelay40c


IDelay40b:

cbi		PortB,0		;off red and green LEDs
cbi		PortB,1

rcall	DetectCntTwo	;new function service is required from button


IDelay40c:

pop     r31
pop     r0

ret

;--------------------------


IDelay20th:

push	r16
ldi		r16,2


IDelay20th1:

rcall	IDelay40th
dec		r16
brne	IDelay20th1

pop		r16
ret

;----------------------------


IDelay10th:

push	r16
ldi		r16,4


IDelay10th1:

rcall	IDelay40th
dec		r16
brne	IDelay10th1

pop		r16
ret

;-----------------------------


IDelayQuarter:

push	r16
ldi		r16,10			;quarter second delay


IDelayQuarter1:

rcall	IDelay40th
dec		r16
brne	IDelayQuarter1

pop		r16
ret

;-------------------------


IDelayHalf:

push	r16
ldi		r16,20


IDelayHalf1:

rcall	IDelay40th
dec		r16
brne	IDelayHalf1

pop		r16
ret

;--------------------------


Mult100:		;Multiplies value in R20 by 100 and returns value in R20

push	r1
clr		r1	;zero r1 for upcoming data

tst		r20
breq	Mult100b

ldi		r16,100	;the multiplication value


Mult100a:

add		r1,r16
dec		r20
brne	Mult100a


Mult100b:

mov		r20,r1	;for return of value
pop		r1
ret


;--------------------------------------


Mult10:		; multiplies value in R20 by 10 returning value in R20

push 	r1
clr		r1		;zero R1

tst		R20		;test long count value
breq	Mult10b		;value is zero

ldi		r16,10		;put ten into r16


Mult10a:

add		r1,r16		;add ten for each value in R1
dec		r20
brne	Mult10a


Mult10b:

mov		r20,r1		;move r1 value into r16
pop		r1
ret

;-----------------------------


GrabTime:		;grabs and stores time into command memory

push	r25		;scatch
push	r26		;pointer

lds		r26,$67		;get bookmark for command memory pointer
					; put in "X" register

cpi		r26,$B0		;check for end of memory storage
brsh	GTEnd1		;exit if count is same or higher


bclr	7		;disable global interrupts

lds		r16,$61		;grab low time byte
st		x+,r16		;store in command memory
lds		r16,$62		;grab high time byte

lds		r25,$63		;grab day of week
lsl		r25
lsl		r25
lsl		r25		;move day of week into position

or		r16,r25		;add day of week info into r16
st		x+,r16		;store in command memory

bset	7		;enable global interrupts

sts		$67,r26		;store away bookmark

lds		r25,$6B		;get value of GrabTime Flag

cpi		r25,0		
breq	GT1

cpi		r25,1
breq	GT2

cpi		r25,2
breq	GT3


GT4:			;last choice - insert variable day command

ld		r16,-x	;pre decrement x and grab high byte
andi	r16,7	;save only bits 0,1,2
ori		r16,128	;set bit 7 (128) for variable day command
st		x+,r16	;put back in memory and increment x register
rjmp	GTEnd


GTEnd1:		;long red LED flash if no more memory available

sbi		PORTB,0		;on LED
rcall	Delay10th	
cbi		PORTB,0		;off LED


GTEnd:

pop		r26
pop		r25
ret


GT1:		;It is an every day command
		
ld		r16,-x	;pre decrement x and grab high byte
andi	r16,7	;mask out day of week information - save bits 0,1,2 only
st		x+,r16	;put value back into command memory and increment x register

rjmp	GTEnd


GT2:		;it is on a given day-of-week command

rjmp	GTEnd	;leave unchanged since day of week info already there


GT3:		;it is an every-other-day command

ld		r16,-x	;pre decrement x and grab high byte
andi	r16,7	;save only bits 0,1,2
ori		r16,64	;set bit 6 (64) for every-other-day command
st		x+,r16	;store back and increment x register

rjmp	GTEnd


;--------------------------------------------


OnTimes:		;sets a series of on times spread 10 minutes apart
			; starting at 8:00 p.m. -- SUMMER series

rcall	ZeroCommandMemory

push	r26		
push	r28
push	r29

ldi		r26,$6C		;start of command memory store area
ldi		r16,10		;for 10 storage locations

ldi		r29,4		;start at 8:00 p.m.
ldi		r28,176	


TOTa:

st		x+,r28
st		x+,r29

adiw	r28,19		;add 19 minutes to word r28,r29

dec		r16

brne	TOTa

rjmp	TOT2b		;Out, pop, and ret

;-------------------------------


OnTimes2:		;sets a series of on times spread 17 minutes apart
			; starting at 6:00 p.m.-- WINTER series

rcall	ZeroCommandMemory

push	r26		
push	r28
push	r29

ldi		r26,$6C		;start of command memory store area
ldi		r16,10		;for 10 storage locations

ldi		r29,4		;start at 6:00 p.m.
ldi		r28,56	


TOT2a:

st		x+,r28
st		x+,r29

adiw	r28,29		;add 29 minutes to word r28,r29

dec		r16

brne	TOT2a


TOT2b:

sts		$67,r26		;store bookmark

pop		r29
pop		r28
pop		r26

ret

;-------------------------------


ManualProgramEntry:

			;manually enters time commands into command store

push	r25		;scratch
push	r26		;push X register
push	r27		
push	r28


MPEs:

lds		r26,$67		;get bookmark for command memory pointer
			; put in "X" register

cpi		r26,$B0		;check for end of memory storage
brsh	MPE1		;exit if count is same or higher
rjmp	MPE2		;entry okay, ... continue


MPE1:			;give long RED flash showing memory is full

sbi		PORTB,0		;on LED
rcall	Delay10th	
cbi		PORTB,0		;off LED

rjmp	MPEEnd



MPE2:

ldi		r16,6			;set up for SIX green flashes
rcall	FlashGLED		;indicate that six entries are needed

rcall	Button3Delay	;Enter MSD hours value
rcall	Mult10			;multiply r20 value by TEN for MSD

mov		r25,r20			;save r20 MSD value in r25

rcall	OneGreenFlash	;flash Green LED to indicate another entry needed

rcall	Button3Delay	;enter LSD hours value
add		r25,r20			;add LSD hours to r25 value

clr		r26
clr		r27		;zero r26 and r27 (X register)


MPE2a:

tst		r25
breq	MPE2b

adiw	r26,60		;add value of 60 to x reigster pair
dec		r25	
rjmp	MPE2a


MPE2b:			;hours converted to minutes at this point

rcall	TwoGreenFlash	;flash Green LED twice to indicate another 
						; set of MSD, LSD entries needed

rcall	Button3Delay	;get MSD of minutes
rcall	Mult10			;multiply value by 10
add		r26,r20			;add MSD X 10 to r26
brcs	MPE2b1			;branch if carry is set
rjmp	MPE2b2


MPE2b1:

inc		r27		;add one to highest byte of x register pair


MPE2b2:


rcall	OneGreenFlash	;flash green LED indicating another entry is needed

rcall	Button3Delay	;get LSD of minutes
add		r26,r20			;add LSD to r26
brcs	MPE2b3			;branch if carry is set
rjmp	MPE2b4	


MPE2b3:

inc		r27		;add one to MS byte of x register pair	


MPE2b4:					;correct value in minutes in x register pair

andi	r27,7			;save only lowest time digits in event of entry error

rcall	OneGreenFlash	;indicate another entry is needed

rcall	Button3Delay	;get Day-of-week or other command entry
cpi		r20,8
brsh	MPE5			;branch is same or higher for other process


MPE4:

lsl		r20
lsl		r20
lsl		r20		;move day-of-week or every day (0) command into position

or		r27,r20		;add data to MS byte of x register pair


MPE8:		;add data to command memory store and increment 
			;  bookmark pointer

bclr	7		;off interrupts

lds		r28,$67		;grab bookmark and put in y register

st		y+,r26
st		y+,r27		;store values into command store area

sts		$67,r28		;save bookmark

bset	7		;turn interrupts back on



MPE9:

rcall	OneGreenFlash	;indicate another entry is needed

rcall	Button3Delay	;enter whether to continue (0) or exit (1 or higher)

cpi		r20,0			;go for another if zero
breq	MPEs			;go for another time entry
rjmp	MPEEnd


MPE5:

cpi		r20,8		
breq	MPE7		;it is every-other-day command


MPE6:			;it is a variable day command

ori		r27,128
rjmp	MPE8		;set bit for variable day command


MPE7:			;it is every-other-day command

ori		r27,64		;set bit for every-other-day
rjmp	MPE8



MPEEnd:

pop		r28
pop		r27
pop		r26
pop		r25

ret

;----------------------------------------

ReadEEPROM:


bclr	7			;off global interuupts

push	r26			;"X" register

ldi		r16,0			;start of EEPROM data
rcall	ReadEP
inc		r16			;got to next address location
sts		$67,R19		;put bookmark into SRAM
ldi		r26,$6C		;get "X" register set up at beginning of SRAM



ReadE1:

rcall	ReadEP		;get new EEPROM byte for store in SRAM
st		x+,r19		;store and post increment X register
inc		r16			;go to next location in EEPROM
cpi		r16,70
brsh	ReadEnd

rjmp	ReadE1		;repeat loop


ReadEnd:

pop		r26
bset	7			;On global interrupts
ret


;-----------------------------------------

WriteEEPROM:	;R16 address in, R19 data out

bclr	7		;Off global interrupts

push	r26		;X register
lds		r19,$67	;Store bookmark
ldi		r16,0
rcall	WriteEP
inc		r16
ldi		r26,$6C	;start of SRAM


WriteE1:

ld		r19,x+	;load SRAM byte
rcall	WriteEP
inc		r16
cpi		r16,70
brsh	WriteEEnd
rjmp	WriteE1


WriteEEnd:

pop		r26
bset	7		;On global interrupts
ret


;----------------------------------------

ReadEP:				;R16 data in, R19 data out


sbic	EECR,EEWE
rjmp	ReadEP

out		EEAR,r16
sbi		EECR,EERE

in		r19,EEDR
ret


;-------------------------------------

WriteEP:			;r16 = address, r19 = data

sbic	EECR,EEPE
rjmp	WriteEP

out		EEAR,r16	;out address

out		EEDR,r19	;out data
sbi		EECR,EEMPE	;out one to EEMPE
sbi		EECR,EEPE

ret

;---------------------------


ToggleRelayRecord:	;Long hold down will put GrabTim4 in record mode
					;  (toggles relay when entering this routine)
push	r16
push	r20
clr		r16
clr		r20
sts		$68,r20		;clear storage location for delay on time in min.

rcall	ToggleAC


TRR1:

rcall	Delay40th
inc		r16
cpi		r16,120
brne	TRR3

sbr		r18,8			;set grabtime RECORD flag on, r18,3(8)
rcall	OffRelay

clr		r16
sts		$68,r16			;clear minutes in on time store


TRR2:

sbi		PortB,0			;on red LED
rcall	Delay40th	
cbi		PortB,0			;off red LED
rcall	Delay10th	

sbis	PIND,2			;skip if PIND,2(4) is set, switch up
rjmp	TRR2


TRREnd:

pop		r20
pop		r16
ret


TRR3:

sbis	PIND,2			;skip if PIND,2(4) is set, switch up
rjmp	TRR1


TRR4:

clr		r16


TRR5:

inc		r16
rcall	Delay40th

sbic	PIND,2		;skip if PIND2(4) is clear (switch down)
rjmp	TRR6


TRR7:

clr		r16			;zero switch down counter


TRR7a:

rcall	Delay40th

sbic	PIND,2		;skip if PIND2(4) is low (switch down)
rjmp	TRR7b		;jump out
inc		r16
rjmp	TRR7a


TRR7b:

cpi		r16,15
brsh	TRR7d


TRR7e:

inc		r20			;set 10 minute time in r20 (one count)
rjmp	TRR4


TRR7d:

inc		r20
inc		r20
inc		r20
inc		r20
inc		r20			;set 50 minutes time in r20 (5 counts)
rjmp	TRR7e


TRR6:

cpi		r16,31 		;3/4 second delay count
brlo	TRR5


TRR8:

cpi		r20,0
breq	TRREnd

clr		r22			;zero ten counter
sts		$68,r20		;store away value

rjmp	TRREnd

;-------------------------------------------
